@@ -162,13 +162,15 @@ class DavSession(object):
     '''
 
     def __init__(self, url, username='', password='', verify=True, auth=None,
-                 useragent=USERAGENT, dav_header=None):
+                 useragent=USERAGENT, verify_fingerprint=None,
+                 dav_header=None):
         if username and not password:
             password = utils.get_password(username, url)
 
         self._settings = {
             'verify': prepare_verify(verify),
-            'auth': prepare_auth(auth, username, password)
+            'auth': prepare_auth(auth, username, password),
+            'verify_fingerprint': verify_fingerprint,
         }
         self.useragent = useragent
         self.url = url.rstrip('/') + '/'
@@ -222,6 +224,8 @@ class DavStorage(Storage):
     :param password: Password for authentication.
     :param verify: Verify SSL certificate, default True. This can also be a
         local path to a self-signed SSL certificate.
+    :param verify_fingerprint: Optional. SHA1 or MD5 fingerprint of the
+        expected server certificate.
     :param auth: Optional. Either ``basic``, ``digest`` or ``guess``. Default
         ``guess``. If you know yours, consider setting it explicitly for
         performance.
@@ -246,17 +250,18 @@ class DavStorage(Storage):
     _session = None
     _repr_attributes = ('username', 'url')
 
-    def __init__(self, url, username='', password='', collection=None,
-                 verify=True, auth=None, useragent=USERAGENT,
-                 unsafe_href_chars='@', **kwargs):
+    def __init__(self, url, username='', password='', verify=True, auth=None,
+                 useragent=USERAGENT, unsafe_href_chars='@',
+                 verify_fingerprint=None, **kwargs):
         super(DavStorage, self).__init__(**kwargs)
 
         url = url.rstrip('/') + '/'
+        collection = kwargs.get('collection')
         if collection is not None:
             url = utils.urlparse.urljoin(url, collection)
         self.session = DavSession(url, username, password, verify, auth,
-                                  useragent, dav_header=self.dav_header)
-        self.collection = collection
+                                  useragent, verify_fingerprint,
+                                  dav_header=self.dav_header)
         self.unsafe_href_chars = unsafe_href_chars
 
         # defined for _repr_attributes
@@ -268,7 +273,8 @@ class DavStorage(Storage):
         if kwargs.pop('collection', None) is not None:
             raise TypeError('collection argument must not be given.')
         discover_args, _ = utils.split_dict(kwargs, lambda key: key in (
-            'username', 'password', 'verify', 'auth', 'useragent'
+            'username', 'password', 'verify', 'auth', 'useragent',
+            'verify_fingerprint',
         ))
         d = cls.discovery_class(DavSession(
             url=url, dav_header=cls.dav_header, **discover_args))
@@ -329,8 +335,12 @@ class DavStorage(Storage):
             try:
                 hrefs_left.remove(href)
             except KeyError:
-                raise KeyError('{} doesn\'t exist in {}'
-                               .format(href, hrefs_left))
+                if href in hrefs:
+                    dav_logger.warning('Server sent item twice: {}'
+                                       .format(href))
+                else:
+                    dav_logger.warning('Server sent unsolicited item: {}'
+                                       .format(href))
         for href in hrefs_left:
             raise exceptions.NotFoundError(href)
         return rv
@@ -404,10 +414,20 @@ class DavStorage(Storage):
         )
         root = etree.XML(response.content)
         for element in root.iter('{DAV:}response'):
-            etag = element.find('{DAV:}propstat').find(
-                '{DAV:}prop').find('{DAV:}getetag').text
+            propstat = element.find('{DAV:}propstat')
+            prop = propstat.find('{DAV:}prop')
+
+            contenttype = prop.find('{DAV:}getcontenttype').text
+
+            etag = prop.find('{DAV:}getetag').text
             href = self._normalize_href(element.find('{DAV:}href').text)
-            yield href, etag
+            if self.item_mimetype not in contenttype:
+                dav_logger.debug(
+                    'Skipping item with href {!r} '
+                    'because content type {!r} != {!r}.'
+                    .format(href, contenttype, self.item_mimetype))
+            else:
+                yield href, etag
 
 
 class CaldavStorage(DavStorage):
@@ -514,6 +534,7 @@ class CaldavStorage(DavStorage):
             <C:calendar-query xmlns:D="DAV:"
                 xmlns:C="urn:ietf:params:xml:ns:caldav">
                 <D:prop>
+                    <D:getcontenttype/>
                     <D:getetag/>
                 </D:prop>
                 <C:filter>
