@@ -37,14 +37,13 @@ class FilesystemStorage(Storage):
     storage_name = 'filesystem'
     _repr_attributes = ('path',)
 
-    def __init__(self, path, fileext, collection=None, encoding='utf-8',
-                 create=True, **kwargs):
+    def __init__(self, path, fileext, encoding='utf-8', create=True, **kwargs):
         super(FilesystemStorage, self).__init__(**kwargs)
         path = expand_path(path)
+        collection = kwargs.get('collection')
         if collection is not None:
             path = os.path.join(path, collection)
         checkdir(path, create=create)
-        self.collection = collection
         self.path = path
         self.encoding = encoding
         self.fileext = fileext
@@ -62,7 +61,9 @@ class FilesystemStorage(Storage):
         return os.path.join(self.path, href)
 
     def _get_href(self, item):
-        return item.ident + self.fileext
+        # XXX: POSIX only defines / and \0 as invalid chars, but we should make
+        # this work crossplatform.
+        return item.ident.replace('/', '_') + self.fileext
 
     def list(self):
         for fname in os.listdir(self.path):
