@@ -54,6 +54,8 @@ class HttpStorage(Storage):
     :param password: Password for authentication.
     :param verify: Verify SSL certificate, default True. This can also be a
         local path to a self-signed SSL certificate.
+    :param verify_fingerprint: Optional. SHA1 or MD5 fingerprint of the
+        expected server certificate.
     :param auth: Optional. Either ``basic``, ``digest`` or ``guess``. Default
         ``guess``. If you know yours, consider setting it explicitly for
         performance.
@@ -80,8 +82,8 @@ class HttpStorage(Storage):
     _repr_attributes = ('username', 'url')
     _items = None
 
-    def __init__(self, url, username='', password='', collection=None,
-                 verify=True, auth=None, useragent=USERAGENT, **kwargs):
+    def __init__(self, url, username='', password='', verify=True, auth=None,
+                 useragent=USERAGENT, verify_fingerprint=None, **kwargs):
         super(HttpStorage, self).__init__(**kwargs)
 
         if username and not password:
@@ -89,17 +91,18 @@ class HttpStorage(Storage):
 
         self._settings = {
             'verify': prepare_verify(verify),
+            'verify_fingerprint': verify_fingerprint,
             'auth': prepare_auth(auth, username, password),
             'latin1_fallback': False
         }
         self.username, self.password = username, password
         self.useragent = useragent
 
+        collection = kwargs.get('collection')
         if collection is not None:
             url = urlparse.urljoin(url, collection)
         self.url = url
         self.parsed_url = urlparse.urlparse(self.url)
-        self.collection = collection
 
     def _default_headers(self):
         return {'User-Agent': self.useragent}
